/**
 * lazyload组件
 */

(function ($) {

    var lazyload = {
        init: function (opt) {
            this.img.init(opt);
        },
        img: {
            init: function (opt) {
                var that = this,
                    win = window;
                if (that.isload) {  //已经初始化过，就触发trigger
                    that.trigger();
                    return;
                }
                var op = {
                        lazyHeight: 200, //预加载当前屏幕以下lazyHeight内的图片
                        definition: false,  //true表示retina需要显示size大小的图片
                        size: null,  //图片尺寸大小，默认直接去掉后缀使用原图大小，支持多种DPI{'1.5' : '120x120' , '2' : '180x180'}
                        effect: "fadeIn"
                    },
                    opt = opt || {};
                $.extend(that, op, opt);
                var definition = that.definition,
                    devicePixelRatio = win.devicePixelRatio;
                that.definition = definition && devicePixelRatio && devicePixelRatio > 1 || false;  //配置true且devicePixelRatio大于1，部分安卓机器是1.5的
                that.DPR = devicePixelRatio;
                var minDist = 5,
                    minTime = 200,  //单位ms
                    isPhone = that.isPhone = that.fetchVersion();
                if (isPhone) {  //针对ios6以下的版本,后退键回到页面不触发onscroll时，直接使用touch事件替代
                    var touchLazy,
                        timerPhone;
                    $(win).on('touchstart', function (ev) {
                        touchLazy = {
                            sy: win.pageYOffset,
                            time: Date.now()
                        };
                        timerPhone && clearTimeout(timerPhone);
                    }).on('touchend', function (ev) {
                        if (ev && ev.changedTouches) {
                            var disty = Math.abs(win.pageYOffset - touchLazy.sy);
                            if (disty > minDist) {/* && disty > distx*/
                                var timedist = Date.now() - touchLazy.time;
                                timerPhone = setTimeout(function () {
                                    that.changeimg();
                                    touchLazy = {};
                                    clearTimeout(timerPhone);
                                    timerPhone = null;
                                }, timedist > minTime ? 0 : 200);
                            }
                        }
                        else {  //trigger触发
                            that.changeimg();
                        }
                    }).on('touchcancel', function () {
                        timerPhone && clearTimeout(timerPhone);
                        touchLazy = null;
                    });
                }
                else {
                    $(win).on('scroll', function () {
                        that.changeimg();
                    });
                }
                that.trigger();
                that.isload = true;

            },
            trigger: function () {
                var that = this,
                    isPhone = that.isPhone,
                    eventType = isPhone && 'touchend' || 'scroll';
                if (that['imglist']) {
                    that['imglist'].each(function (n, node) {
                        node && (node.onerror = node.onload = null);
                    });
                }
                that['imglist'] = $('.article-content img');
                $(window).trigger(eventType);
            },
            fetchVersion: function () {
                var systemVer = navigator.appVersion.match(/(iPhone\sOS)\s([\d_]+)/),
                    isPhone = systemVer && true || false,
                    version = isPhone && systemVer[2].split('_');
                version = version && parseFloat(version.length > 1 ? version.splice(0, 2).join('.') : version[0], 10);
                return isPhone && version < 6;
            },
            setImgSrc: function (url, size) {
                if (!url) return;
                size = size || this.size;
                size = size && (typeof size == 'string' ? size : size[this.DPR]) || null;
                size && (size = ['_' , size , '.jpg'].join(''));
                var arr = url.lastIndexOf('_.'),  //查找最后一个，url中可能存在_.
                    last = arr != -1 ? url.slice(arr + 2) : null,  //取到_.后的字符串
                    isWebp = last && last.toLowerCase() == 'webp' ? true : false,  //是否webp
                    newurl = isWebp ? url.slice(0, arr) : url,
                    src = newurl.replace(/_\d+x\d+\.jpg?/g, '');  //去掉存在的后缀_100x100.jpg
                src += size;
                return isWebp && (src + '_.webp') || src;
            },
            changeimg: function () {
                var that = this,
                    win = window,
                    lazyHeight = that.lazyHeight,
                    definition = that.definition;

                function inViewport(el) {
                    var top = win.pageYOffset,
                        btm = top + win.innerHeight,
                        elTop = el.offset().top;
                    return elTop >= top && elTop - lazyHeight <= btm;
                }

                function act(_self, n) {
                    var original = _self.attr('data-ks-lazyload'),
                        postfix = _self.attr('postfix');
                    //console.log("original:"+original+"---postfix:"+postfix);
                    if (!original) return;
                    var src = original + postfix;
                    // definition && (original = that.setImgSrc(src));
                    _self.attr('src', src).addClass(that.effect);
                    _self.addClass('content-pic');
                    if (!_self[0].onload) {
                        _self[0].onload = _self[0].onerror = function () {
                            $(this).removeClass('lazy').removeAttr('data-ks-lazyload');
                            that['imglist'][n] = null;
                            this.onerror = this.onload = null;
                        }
                    }
                }

                that['imglist'].each(function (index, node) {
                    if (!node) return;
                    var $this = $(node);
                    //if(!inViewport($this)) return;
                    act($this, index);
                });
            }
        }
    };

    $(function () {
        lazyload.init();
    });

})(Zepto);
